@@ -29,6 +29,7 @@
 #include "../document/document_glom.h"
 #include "../mode_data/box_data_list_related.h"
 #include "layoutwidgetbase.h"
+#include "../mode_data/treestore_layout.h" //Forthe enum.
 #include <map>
 #include <list>
 
@@ -47,9 +48,10 @@ public:
   virtual void set_table(const Glib::ustring& table_name);
 
   /** Add a field.
-   * @param layoutitem_field The layout item that describes this field
+   * @param layoutitem_field The layout item that describes this field,
+   * @param table_name The table on which this layout appears.
    */
-  virtual void add_field(const LayoutItem_Field& layoutitem_field);
+  virtual void add_field(const LayoutItem_Field& layoutitem_field, const Glib::ustring& table_name);
   virtual void remove_field(const Glib::ustring& id);
 
   typedef std::map<int, Field> type_map_field_sequence;
@@ -57,14 +59,13 @@ public:
 
   virtual void add_layout_item(const LayoutItem& group);
   virtual void add_layout_group(const LayoutGroup& group);
-  
 
-  virtual void set_field_editable(const Field& field, bool editable = true);
+  virtual void set_field_editable(const LayoutItem_Field& field, bool editable = true);
 
-  virtual Gnome::Gda::Value get_field_value(const Field& field) const;
-  virtual Gnome::Gda::Value get_field_value(const Glib::ustring& id) const;
-  virtual void set_field_value(const Field& field, const Gnome::Gda::Value& value);
-  virtual void set_field_value(const Glib::ustring& id, const Gnome::Gda::Value& value);
+  virtual Gnome::Gda::Value get_field_value(const LayoutItem_Field& field) const;
+  //virtual Gnome::Gda::Value get_field_value(const Glib::ustring& id) const;
+  virtual void set_field_value(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
+  //virtual void set_field_value(const Glib::ustring& id, const Gnome::Gda::Value& value);
 
 
   typedef std::list<Gtk::Widget*> type_list_widgets;
@@ -85,37 +86,37 @@ public:
   virtual void get_layout_group(LayoutGroup& group);
 
   /** For instance,
-   * void on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& value);
+   * void on_flowtable_field_edited(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
    */
-  typedef sigc::signal<void, const Glib::ustring&, const Gnome::Gda::Value&> type_signal_field_edited;
+  typedef sigc::signal<void, const LayoutItem_Field&, const Gnome::Gda::Value&> type_signal_field_edited;
   type_signal_field_edited signal_field_edited();
 
 
 protected:
 
-  virtual type_list_widgets get_field(const Glib::ustring& id);
-  virtual type_list_const_widgets get_field(const Glib::ustring& id) const;
-
-  virtual type_list_widgets get_field(const Field& field);
-  virtual type_list_const_widgets get_field(const Field& field) const;
+  virtual type_list_widgets get_field(const LayoutItem_Field& field);
+  virtual type_list_const_widgets get_field(const LayoutItem_Field& field) const;
 
   ///Get portals whose relationships have @a from_key as the from_key.
-  virtual type_list_widgets get_portals(const Glib::ustring& from_key);
+  virtual type_list_widgets get_portals(const LayoutItem_Field& from_key);
 
   int get_suitable_width(Field::glom_field_type field_type);
-  void on_entry_edited(const Gnome::Gda::Value& value, Glib::ustring id);
-  void on_flowtable_entry_edited(const Glib::ustring& id, const Gnome::Gda::Value& value);
+
+  void on_entry_edited(const Gnome::Gda::Value& value,  LayoutItem_Field field);
+  void on_flowtable_entry_edited(const LayoutItem_Field& field, const Gnome::Gda::Value& value);
 
   /// Remember the layout widget so we can iterate through them later.
-  void add_layoutwidgetbase(LayoutWidgetBase* layout_widget);
   void on_layoutwidget_changed();
-    
+
+  void on_datawidget_layout_item_added(TreeStore_Layout::enumType item_type, DataWidget* pDataWidget);
+
+
   class Info
   {
   public:
     Info();
-    
-    Field m_field; //Store the field information so we know the title, ID, and type.
+
+    LayoutItem_Field m_field; //Store the field information so we know the title, ID, and type.
     Glib::ustring m_group;
 
     Gtk::Alignment* m_first;
@@ -137,6 +138,12 @@ protected:
   typedef std::list< LayoutWidgetBase* > type_list_layoutwidgets;
   type_list_layoutwidgets m_list_layoutwidgets;
 
+  void add_field_at_position(const LayoutItem_Field& layoutitem_field, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before);
+  void add_layoutwidgetbase(LayoutWidgetBase* layout_widget);
+  void add_layoutwidgetbase(LayoutWidgetBase* layout_widget, const type_list_layoutwidgets::iterator& add_before);
+  void add_layout_item_at_position(const LayoutItem& item, const type_list_layoutwidgets::iterator& add_before);
+  void add_layout_group_at_position(const LayoutGroup& group, const type_list_layoutwidgets::iterator& add_before);
+
   Glib::ustring m_table_name;
 
   type_signal_field_edited m_signal_field_edited;
